@@ -3,7 +3,7 @@ use bevy::prelude::*;
 use bevy_ui_navigation::systems::{
     default_gamepad_input, default_keyboard_input, default_mouse_input, InputMapping,
 };
-use bevy_ui_navigation::{FocusState, Focusable, NavEvent, NavigationPlugin};
+use bevy_ui_navigation::{FocusState, Focusable, NavEvent, NavRequestSystem, NavigationPlugin};
 
 /// This example illustrates how to make a button "lock". To lock the UI, press
 /// 'A' on controller or 'left click' on mouse when the button with the lock is
@@ -20,11 +20,11 @@ fn main() {
         .init_resource::<Images>()
         .init_resource::<InputMapping>()
         .add_startup_system(setup)
-        .add_system(button_system)
-        .add_system(default_keyboard_input)
-        .add_system(default_gamepad_input)
-        .add_system(default_mouse_input)
-        .add_system(print_nav_events)
+        .add_system(button_system.after(NavRequestSystem))
+        .add_system(default_keyboard_input.before(NavRequestSystem))
+        .add_system(default_gamepad_input.before(NavRequestSystem))
+        .add_system(default_mouse_input.before(NavRequestSystem))
+        .add_system(print_nav_events.after(NavRequestSystem))
         .run();
 }
 
