@@ -4,7 +4,8 @@ use bevy_ui_navigation::systems::{
     default_gamepad_input, default_keyboard_input, default_mouse_input, InputMapping,
 };
 use bevy_ui_navigation::{
-    FocusState, Focusable, Focused, NavMarkerPropagationPlugin, NavMenu, NavigationPlugin,
+    FocusState, Focusable, Focused, NavMarkerPropagationPlugin, NavMenu, NavRequestSystem,
+    NavigationPlugin,
 };
 
 macro_rules! column_type {
@@ -51,11 +52,11 @@ fn main() {
         .add_plugin(NavigationPlugin)
         .init_resource::<InputMapping>()
         .add_startup_system(setup)
-        .add_system(button_system)
-        .add_system(default_keyboard_input)
-        .add_system(default_gamepad_input)
-        .add_system(default_mouse_input)
-        .add_system(print_menus)
+        .add_system(button_system.after(NavRequestSystem))
+        .add_system(default_keyboard_input.before(NavRequestSystem))
+        .add_system(default_gamepad_input.before(NavRequestSystem))
+        .add_system(default_mouse_input.before(NavRequestSystem))
+        .add_system(print_menus.after(NavRequestSystem))
         .run();
 }
 
